using System;
using System.Collections.Generic;
using System.Text;

namespace I2CDotNet
{
    interface I2CDriverInterface
    {
        /// <summary>
        /// Sets the I2C slave address of the device that this interface interacts with
        /// </summary>
        /// <param name="address"></param>
        /// <returns>true on success</returns>
        bool SetSlaveAddress(uint address);

        /// <summary>
        /// Write to I2C device
        /// </summary>
        /// <param name="subAddress"> Device internal address</param>
        /// <param name="buffer"> data to be written to the device </param>
        /// <param name="size"> number of bytes to write</param>
        /// <returns></returns>
        bool Write(uint subAddress, byte[] buffer, uint size);

        /// <summary>
        /// Read from I2C device
        /// </summary>
        /// <param name="subAddress">Device internal address</param>
        /// <param name="buffer"> [out] The data will be written to this array</param>
        /// <param name="size"> number of bytes to read </param>
        /// <returns></returns>
        bool Read(uint subAddress, byte[] buffer, uint size);
    }
}
